/*
 * This file is part of the ToolBox.
 * Copyright Thomas Jacob.
 *
 * READ README.TXT BEFORE USE!!
 */


// Main header file
#include <ToolBox\Headers.h>


// --- Class ChecksumCalculator ---

ChecksumCalculator::ChecksumCalculator()
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	for (int i = 0; i < 256; ++i)
	{
		unsigned long c = i<<24;
		for (int j = 0; j < 8; j++)
			c = c & 0x80000000 ? (c << 1) ^ 0x04C11DB7 : (c << 1);
		Crc32Table[i] = c;
	}
}

ChecksumCalculator::~ChecksumCalculator()
{
	ASSERTION_THIS;

	#ifdef _TOOLBOX_TEST
		InstanceCount--;
	#endif
}

unsigned long ChecksumCalculator::GetCrc32(const char * buffer, int len)
{
	ASSERTION_THIS;

	unsigned long crc = 0xFFFFFFFF;
	for (int i=0; i<len; i++)
		crc = (crc << 8) ^ Crc32Table[(crc >> 24) ^ (unsigned char) buffer[i]];

	return ~crc;
}

#ifdef _TOOLBOX_TEST
	int ChecksumCalculator::InstanceCount = 0;
#endif

#ifdef _TOOLBOX_TEST
	void ChecksumCalculator::RunTestSuite(int * performedTests, int * failedTests)
	{
		const char * buffer1 = "Hallo, dies ist ein kleiner Test!\r\n";
		const char * buffer2 = "Und dies ist noch ein anderer, aber etwas grerer Test.\r\n";

		ChecksumCalculator * Calculator;
		if ((Calculator = new ChecksumCalculator) == NULL)
			throw EOutOfMemory::Get();

		if (Calculator->GetCrc32(buffer1, strlen(buffer1)) != 0x678088DA)
			{printf("Test ChecksumCalculator[0] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (Calculator->GetCrc32(buffer1, strlen(buffer1)) != 0x678088DA)
			{printf("Test ChecksumCalculator[1] failed!\n"); (*failedTests)++;}
		(*performedTests)++;

		if (Calculator->GetCrc32(buffer2, strlen(buffer2)) != 0x0D26A227)
			{printf("Test ChecksumCalculator[2] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (Calculator->GetCrc32(buffer2, strlen(buffer2)) != 0x0D26A227)
			{printf("Test ChecksumCalculator[3] failed!\n"); (*failedTests)++;}
		(*performedTests)++;

		delete Calculator;
	}
#endif
