/*
 * This file is part of the ToolBox.
 * Copyright Thomas Jacob.
 *
 * READ README.TXT BEFORE USE!!
 */


#ifndef __TOOLBOX_CONFIG_INL
#define __TOOLBOX_CONFIG_INL


// --- Class Config ---

inline ConfigSection * Config::GetContent()
{
	return Content;
}


// --- Class Config::StringWriter ---

inline const char * Config::StringWriter::GetString()
{
	ASSERTION_THIS;
	return Buffer;
}


// --- Class ConfigSection ---

inline bool ConfigSection::GetBoolValue(const char * name)
{
	ASSERTION_THIS;
	return GetParameter(name)->GetBoolValue();
}

inline double ConfigSection::GetDoubleValue(const char * name)
{
	ASSERTION_THIS;
	return GetParameter(name)->GetDoubleValue();
}

inline int ConfigSection::GetIntValue(const char * name)
{
	ASSERTION_THIS;
	return (int) GetParameter(name)->GetLongValue();
}

inline long ConfigSection::GetLongValue(const char * name)
{
	ASSERTION_THIS;
	return GetParameter(name)->GetLongValue();
}

inline const char * ConfigSection::GetName()
{
	ASSERTION_THIS;
	return Name;
}

inline PointeredList * ConfigSection::GetParameterNames()
{
	ASSERTION_THIS;
	return Parameters->GetKeys();
}

inline PointeredList * ConfigSection::GetSectionNames()
{
	ASSERTION_THIS;
	return Sections->GetKeys();
}

inline PointeredList * ConfigSection::GetSections()
{
	ASSERTION_THIS;
	return Sections->GetValues();
}

inline const char * ConfigSection::GetStringValue(const char * name)
{
	ASSERTION_THIS;
	return GetParameter(name)->GetStringValue();
}

inline int ConfigSection::GetType(const char * name)
{
	ASSERTION_THIS;
	return GetParameter(name)->GetType();
}

inline void ConfigSection::SetBoolValue(const char * name, bool boolValue)
{
	ASSERTION_THIS;
	GetParameter(name)->SetBoolValue(boolValue);
}

inline void ConfigSection::SetDoubleValue(const char * name, double doubleValue)
{
	ASSERTION_THIS;
	GetParameter(name)->SetDoubleValue(doubleValue);
}

inline void ConfigSection::SetIntValue(const char * name, int intValue)
{
	ASSERTION_THIS;
	GetParameter(name)->SetLongValue((long) intValue);
}

inline void ConfigSection::SetLongValue(const char * name, long longValue)
{
	ASSERTION_THIS;
	GetParameter(name)->SetLongValue(longValue);
}

inline void ConfigSection::SetStringValue(const char * name, const char * stringValue)
{
	ASSERTION_THIS;
	GetParameter(name)->SetStringValue(stringValue);
}


// --- Class ConfigParameter ---

inline int ConfigParameter::GetType()
{
	ASSERTION_THIS;
	return Type;
}


#endif
