/*
 * This file is part of the ToolBox.
 * Copyright Thomas Jacob.
 *
 * READ README.TXT BEFORE USE!!
 */


// Main header file
#include <ToolBox\Headers.h>


// --- Class Directory ---

Directory::Directory()
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	if ((Entries = new ContainingList<DirectoryEntry>) == NULL)
		throw EOutOfMemory::Get();
}

Directory::~Directory()
{
	ASSERTION_THIS;

	if (Entries) {delete Entries; Entries = NULL;}

	#ifdef _TOOLBOX_TEST
		InstanceCount--;
	#endif
}

#ifdef _TOOLBOX_TEST
	int Directory::InstanceCount = 0;
#endif

void Directory::ReadFrom(const char * rootWithFilter,
	int requiredAttributes, int forbiddenAttributes, bool recurse)
{
	ASSERTION_THIS;

	Path * RootWithFilterCorrected;
	if ((RootWithFilterCorrected = new Path(rootWithFilter)) == NULL)
		throw EOutOfMemory::Get();

	ReadFromInternal(RootWithFilterCorrected, requiredAttributes,
		forbiddenAttributes, recurse);

	delete RootWithFilterCorrected;
}

void Directory::ReadFrom(const char * root, const char * filter,
	int requiredAttributes, int forbiddenAttributes, bool recurse)
{
	ASSERTION_THIS;

	Path * RootWithFilterCorrected;
	if ((RootWithFilterCorrected = new Path(root)) == NULL)
		throw EOutOfMemory::Get();
	RootWithFilterCorrected->AppendElement(filter);

	ReadFromInternal(RootWithFilterCorrected, requiredAttributes,
		forbiddenAttributes, recurse);

	delete RootWithFilterCorrected;
}

void Directory::ReadFrom(Path * RootWithFilter,
	int requiredAttributes, int forbiddenAttributes, bool recurse)
{
	ASSERTION_THIS;

	Path * RootWithFilterCorrected = RootWithFilter->Clone();

	ReadFromInternal(RootWithFilterCorrected, requiredAttributes,
		forbiddenAttributes, recurse);

	delete RootWithFilterCorrected;
}

void Directory::ReadFrom(Path * Root, const char * filter,
	int requiredAttributes, int forbiddenAttributes, bool recurse)
{
	ASSERTION_THIS;

	Path * RootWithFilterCorrected = Root->Clone();
	RootWithFilterCorrected->AppendElement(filter);

	ReadFromInternal(RootWithFilterCorrected, requiredAttributes,
		forbiddenAttributes, recurse);

	delete RootWithFilterCorrected;
}

void Directory::ReadFromInternal(Path * RootWithFilter,
	int requiredAttributes, int forbiddenAttributes, bool recurse)
{
	ASSERTION_THIS;

	struct _finddata_t findData;
	long findHandle;

	if (recurse)
	{
		PathElement * Filter = RootWithFilter->UnlinkLastElement();

		RootWithFilter->AppendElement("*");
		char * root = RootWithFilter->ToString(false);
		RootWithFilter->DeleteLastElement();

		findHandle = _findfirst(root, &findData);
		if (findHandle != -1)
			do
			{
				if ((findData.attrib & DU_FDIRECTORY)
					&& strcmp(findData.name, ".") != 0
					&& strcmp(findData.name, "..") != 0)
				{
					RootWithFilter->AppendElement(findData.name);
					RootWithFilter->AppendElement(Filter);
					ReadFromInternal(RootWithFilter, requiredAttributes,
						forbiddenAttributes, recurse);
					Filter = RootWithFilter->UnlinkLastElement();
					RootWithFilter->DeleteLastElement();
				}
			} while (_findnext(findHandle, &findData) == 0);

		_findclose(findHandle);
		delete root;

		RootWithFilter->AppendElement(Filter);
	}

	char * root = RootWithFilter->ToString(false);
	findHandle = _findfirst(root, &findData);
	if (findHandle != -1)
		do
		{
			if ((requiredAttributes & ~findData.attrib) == 0
				&& (forbiddenAttributes & findData.attrib) == 0
				&& strcmp(findData.name, ".") != 0
				&& strcmp(findData.name, "..") != 0)
			{
				PathElement * Filter = RootWithFilter->UnlinkLastElement();

				Path * NewPath;
				NewPath = RootWithFilter->Clone();
				NewPath->AppendElement(findData.name);
				DirectoryEntry * NewEntry = new class DirectoryEntry(NewPath);
				delete NewPath;
				if (NewEntry == NULL)
					throw EOutOfMemory::Get();
				AppendEntry(NewEntry);

				RootWithFilter->AppendElement(Filter);
			}
		} while (_findnext(findHandle, &findData) == 0);

	_findclose(findHandle);
	delete root;
}

#ifdef _TOOLBOX_TEST
	void Directory::RunTestSuite(int * performedTests, int * failedTests)
	{
	}
#endif


// --- Class DirectoryEntry ---

DirectoryEntry::DirectoryEntry()
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	InvalidateTime(&AccessTime);
	InvalidateTime(&CreateTime);
	ExistsFlag = false;
	Flags = DU_FNOFLAGS;
	InvalidateTime(&ModifyTime);
	Path = NULL;
	Size = 0UL;
}

DirectoryEntry::DirectoryEntry(class Path * Path)
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	InvalidateTime(&AccessTime);
	InvalidateTime(&CreateTime);
	ExistsFlag = false;
	Flags = DU_FNOFLAGS;
	InvalidateTime(&ModifyTime);
	this->Path = NULL;
	Size = 0UL;

	this->Path = Path->Clone();

	Refresh();
}

DirectoryEntry::DirectoryEntry(const char * path)
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	InvalidateTime(&AccessTime);
	InvalidateTime(&CreateTime);
	ExistsFlag = false;
	Flags = DU_FNOFLAGS;
	InvalidateTime(&ModifyTime);
	Path = NULL;
	Size = 0UL;

	if ((Path = new class Path(path)) == NULL)
		throw EOutOfMemory::Get();

	Refresh();
}

DirectoryEntry::~DirectoryEntry()
{
	ASSERTION_THIS;

	if (Path) {delete Path; Path = NULL;}

	#ifdef _TOOLBOX_TEST
		InstanceCount--;
	#endif
}

DirectoryEntry * DirectoryEntry::Clone()
{
	ASSERTION_THIS;

	DirectoryEntry * Clone;
	if ((Clone = new DirectoryEntry) == NULL)
		throw EOutOfMemory::Get();

	Clone->AccessTime = AccessTime;
	Clone->CreateTime = CreateTime;
	Clone->ExistsFlag = ExistsFlag;
	Clone->Flags = Flags;
	Clone->ModifyTime = ModifyTime;
	Clone->Path = Path->Clone();
	Clone->Size = Size;

	return Clone;
}

long DirectoryEntry::GetAccessTime()
{
	ASSERTION_THIS;

	if (ExistsFlag)
		return (long) mktime(&AccessTime);
	else
		return DU_INVALID;
}

long DirectoryEntry::GetCreateTime()
{
	ASSERTION_THIS;
	return (long) mktime(&CreateTime);
}

long DirectoryEntry::GetModifyTime()
{
	ASSERTION_THIS;

	if (ExistsFlag)
		return (long) mktime(&ModifyTime);
	else
		return DU_INVALID;
}

#ifdef _TOOLBOX_TEST
	int DirectoryEntry::InstanceCount = 0;
#endif

void DirectoryEntry::Invalidate()
{
	ASSERTION_THIS;

	InvalidateTime(&AccessTime);
	InvalidateTime(&CreateTime);
	ExistsFlag = false;
	Flags = DU_FNOFLAGS;
	InvalidateTime(&ModifyTime);
	Size = 0;
}

void DirectoryEntry::InvalidateTime(struct tm * time)
{
	time->tm_sec = DU_INVALID;
	time->tm_min = DU_INVALID;
	time->tm_hour = DU_INVALID;
	time->tm_mday = DU_INVALID;
	time->tm_mon = DU_INVALID;
	time->tm_year = DU_INVALID;
}

void DirectoryEntry::Refresh()
{
	ASSERTION_THIS;

	char * path = GetPath()->ToString(false);
	if (_access(path, 0) != 0)
	{
		delete path;
		Invalidate();
		return;
	}

	struct _finddata_t findData;
	long findHandle;

	findHandle = _findfirst(path, &findData);
	if (findHandle == -1)
	{
		delete path;
		Invalidate();
		return;
	}

	struct tm * time = localtime(&findData.time_access);
	if (time)
	{
		AccessTime = *time;
		AccessTime.tm_mon++;
	}
	else
		InvalidateTime(&AccessTime);

	time = localtime(&findData.time_create);
	if (time)
	{
		CreateTime = *time;
		CreateTime.tm_mon++;
	}
	else
		InvalidateTime(&CreateTime);

	ExistsFlag = true;
	Flags = findData.attrib;

	time = localtime(&findData.time_write);
	if (time)
	{
		ModifyTime = *time;
		ModifyTime.tm_mon++;
	}
	else
		InvalidateTime(&ModifyTime);

	Size = findData.size;

	_findclose(findHandle);
	delete path;
}

#ifdef _TOOLBOX_TEST
	void DirectoryEntry::RunTestSuite(int * performedTests, int * failedTests)
	{
	}
#endif


// --- Class Path ---

Path::Path()
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	Elements = NULL;
	Root = NULL;

	if ((Elements = new ContainingList<PathElement>) == NULL)
		throw EOutOfMemory::Get();
	SetRoot("");
}

Path::Path(const char * path)
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	Elements = NULL;
	Root = NULL;

	if ((Elements = new ContainingList<PathElement>) == NULL)
		throw EOutOfMemory::Get();

	char * pathCopy;
	if ((pathCopy = new char[strlen(path) + 1]) == NULL)
	{
		delete Elements;
		throw EOutOfMemory::Get();
	}
	strcpy(pathCopy, path);

	char * colonPtr;
	char * slashPtr;
	char * elementsPtr;
	if (pathCopy[0] == '\\' && pathCopy[1] == '\\')
	{
		if ((slashPtr = strchr(&pathCopy[2], '\\')) != NULL)
			elementsPtr = &slashPtr[1];
		else
			elementsPtr = &pathCopy[strlen(pathCopy)];
	}
	else
	{
		colonPtr = strchr(pathCopy, ':');
		slashPtr = strchr(pathCopy, '\\');

		if (colonPtr != NULL)
		{
			if (slashPtr == &colonPtr[1])
				elementsPtr = &slashPtr[1];
			else
				elementsPtr = &colonPtr[1];
		}
		else if (pathCopy[0] == '\\')
			elementsPtr = &pathCopy[1];
		else
			elementsPtr = pathCopy;
	}

	char merk = elementsPtr[0];
	elementsPtr[0] = 0;
	SetRoot(pathCopy);
	elementsPtr[0] = merk;

	while (elementsPtr)
	{
		char * nextPtr = strchr(elementsPtr, '\\');
		if (nextPtr)
		{
			nextPtr[0] = 0;
			if (elementsPtr[0])
				AppendElement(elementsPtr);
			nextPtr[0] = '\\';

			elementsPtr = &nextPtr[1];
		}
		else
		{
			if (elementsPtr[0])
				AppendElement(elementsPtr);

			break;
		}
	}

	delete pathCopy;
}

Path::~Path()
{
	ASSERTION_THIS;

	if (Elements != NULL) {delete Elements; Elements = NULL;}
	if (Root != NULL) {delete Root; Root = NULL;}

	#ifdef _TOOLBOX_TEST
		InstanceCount--;
	#endif
}

void Path::AppendPath(const char * path)
{
	ASSERTION_THIS;

	class Path * Path;
	if ((Path = new class Path(path)) == NULL)
		throw EOutOfMemory::Get();
	AppendPath(Path);
	delete Path;
}

void Path::AppendPath(class Path * Path)
{
	ASSERTION_THIS;

	PathElement * Element = Path->GetFirstElement();
	while (Element)
	{
		AppendElement(Element->GetName());
		Element = Element->GetNextElement();
	}
}

bool Path::Contains(const char * path)
{
	return Contains(&Path(path));
}

bool Path::Contains(class Path * Path)
{
	if (stricmp(Root, Path->GetRoot()) != 0)
		return false;

	int pathElementCount = Path->GetElementCount();
	if (pathElementCount > GetElementCount())
		return false;

	for (int i=0; i<pathElementCount; i++)
		if (stricmp(Path->GetElement(i)->GetName(), GetElement(i)->GetName()) != 0)
			return false;

	return true;
}

Path * Path::Clone()
{
	ASSERTION_THIS;

	char * path = ToString(false);
	Path * Clone;
	if ((Clone = new Path(path)) == NULL)
		throw EOutOfMemory::Get();
	delete path;

	return Clone;
}

Path * Path::CreateApplicationFolderPath()
{
	Path * ApplicationPath;
	if ((ApplicationPath = new Path(__argv[0])) == NULL)
		throw EOutOfMemory::Get();

	ApplicationPath->DeleteLastElement();

	return ApplicationPath;
}

Path * Path::CreateApplicationPath()
{
	Path * ApplicationPath;
	if ((ApplicationPath = new Path(__argv[0])) == NULL)
		throw EOutOfMemory::Get();
	return ApplicationPath;
}

Path * Path::CreateRelativePathTo(const char * path)
{
	ASSERTION_THIS;

	class Path * Path;
	if ((Path = new class Path(path)) == NULL)
		throw EOutOfMemory::Get();
	class Path * RelativePath = CreateRelativePathTo(Path);
	delete Path;

	return RelativePath;
}

class Path * Path::CreateRelativePathTo(class Path * Path)
{
	ASSERTION_THIS;

	if (stricmp(Root, Path->GetRoot()) == 0)
	{
		// Both paths are relative, or both are absolute with the same root.
		// So, let's create a relative path.

		// First, we find the common path.
		PathElement * ThisCommonElement;
		PathElement * OtherCommonElement;
		if (GetFirstElement() != NULL && Path->GetFirstElement() != NULL
			&& stricmp(GetFirstElement()->GetName(), Path->GetFirstElement()->GetName()) == 0)
		{
			ThisCommonElement = GetFirstElement();
			OtherCommonElement = Path->GetFirstElement();

			while (ThisCommonElement->GetNextElement() != NULL
				&& OtherCommonElement->GetNextElement() != NULL
				&& stricmp(ThisCommonElement->GetNextElement()->GetName(),
				OtherCommonElement->GetNextElement()->GetName()) == 0)
			{
				ThisCommonElement = ThisCommonElement->GetNextElement();
				OtherCommonElement = OtherCommonElement->GetNextElement();
			}
		}
		else
		{
			ThisCommonElement = NULL;
			OtherCommonElement = NULL;
		}

		// Now build the relative path
		class Path * RelativePath;
		if ((RelativePath = new class Path) == NULL)
			throw EOutOfMemory::Get();

		PathElement * CurrentElement = GetLastElement();
		while (CurrentElement != ThisCommonElement)
		{
			RelativePath->AppendElement("..");
			CurrentElement = CurrentElement->GetPrevElement();
		}

		if (OtherCommonElement == NULL)
			CurrentElement = Path->GetFirstElement();
		else
			CurrentElement = OtherCommonElement->GetNextElement();

		while (CurrentElement != NULL)
		{
			RelativePath->AppendElement(CurrentElement->Clone());
			CurrentElement = CurrentElement->GetNextElement();
		}

		return RelativePath;
	}
	else if (Path->IsAbsolute())
	{
		// Both paths are absolute with a different root
		// or this path is relative and the other one is absolute.
		// So, let's return a clone of the other (absolute) path.

		return Path->Clone();
	}
	else
	{
		// This path is absolute and the other one is relative.
		// The path cannot be created, we return NULL.
		return NULL;
	}
}

class Path * Path::CreateTargetPath(const char * path)
{
	ASSERTION_THIS;

	class Path * Path;
	if ((Path = new class Path(path)) == NULL)
		throw EOutOfMemory::Get();
	class Path * TargetPath = CreateTargetPath(Path);
	delete Path;

	return TargetPath;
}

class Path * Path::CreateTargetPath(class Path * Path)
{
	ASSERTION_THIS;

	if (Path->IsAbsolute())
		return Path->Clone();
	else
	{
		class Path * TargetPath = Clone();

		PathElement * CurrentPathElement = Path->GetFirstElement();
		while (CurrentPathElement != NULL)
		{
			const char * name = CurrentPathElement->GetName();
			if (strcmp(name, "..") == 0)
			{
				if (!TargetPath->IsEmpty())
					TargetPath->DeleteLastElement();
			}
			else if (strcmp(name, ".") != 0)
				TargetPath->AppendElement(name);

			CurrentPathElement = CurrentPathElement->GetNextElement();
		}

		return TargetPath;
	}
}

bool Path::Exists()
{
	ASSERTION_THIS;

	char * path = ToString();
	bool exists = _access(path, 0) == 0;
	delete path;
	return exists;
}

#ifdef _TOOLBOX_TEST
	int Path::InstanceCount = 0;
#endif

bool Path::MakeDirectory()
{
	ASSERTION_THIS;

	char * path = ToString(false);
	bool error = _mkdir(path) != 0;

	delete path;
	return error;
}

void Path::PrependPath(const char * path)
{
	ASSERTION_THIS;

	class Path * Path;
	if ((Path = new class Path(path)) == NULL)
		throw EOutOfMemory::Get();
	PrependPath(Path);
	delete Path;
}

void Path::PrependPath(class Path * Path)
{
	ASSERTION_THIS;

	PathElement * Element = Path->GetLastElement();
	while (Element)
	{
		PrependElement(Element->GetName());
		Element = Element->GetPrevElement();
	}

	SetRoot(Path->GetRoot());
}

#ifdef _TOOLBOX_TEST
	void Path::RunTestSuite(int * performedTests, int * failedTests)
	{
		Path * TestPath;

		TestPath = new Path("C:\\");
		if (strcmp(TestPath->GetRoot(), "C:\\") != NULL) {printf("Test Path[0] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (TestPath->GetElementCount() != 0) {printf("Test Path[1] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		delete TestPath;

		TestPath = new Path("C:\\Test");
		if (strcmp(TestPath->GetRoot(), "C:\\") != NULL) {printf("Test Path[2] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (TestPath->GetElementCount() != 1)
		{
			printf("Test Path[3] failed!\n"); (*failedTests)++;
		}
		else
		{
			if (strcmp(TestPath->GetElement(0)->GetName(), "Test") != NULL) {printf("Test Path[4] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
		}
		(*performedTests)++;
		delete TestPath;

		TestPath = new Path("C:\\Test\\Keks");
		if (strcmp(TestPath->GetRoot(), "C:\\") != NULL) {printf("Test Path[5] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (TestPath->GetElementCount() != 2)
		{
			printf("Test Path[6] failed!\n"); (*failedTests)++;
		}
		else
		{
			if (strcmp(TestPath->GetElement(0)->GetName(), "Test") != NULL) {printf("Test Path[7] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (strcmp(TestPath->GetElement(1)->GetName(), "Keks") != NULL) {printf("Test Path[8] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
		}
		(*performedTests)++;
		delete TestPath;

		TestPath = new Path("C:");
		if (strcmp(TestPath->GetRoot(), "C:") != NULL) {printf("Test Path[9] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (TestPath->GetElementCount() != 0) {printf("Test Path[10] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		delete TestPath;

		TestPath = new Path("C:Test");
		if (strcmp(TestPath->GetRoot(), "C:") != NULL) {printf("Test Path[11] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (TestPath->GetElementCount() != 1)
		{
			printf("Test Path[12] failed!\n"); (*failedTests)++;
		}
		else
		{
			if (strcmp(TestPath->GetElement(0)->GetName(), "Test") != NULL) {printf("Test Path[13] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
		}
		(*performedTests)++;
		delete TestPath;

		TestPath = new Path("C:Test\\Keks");
		if (strcmp(TestPath->GetRoot(), "C:") != NULL) {printf("Test Path[14] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (TestPath->GetElementCount() != 2)
		{
			printf("Test Path[15] failed!\n"); (*failedTests)++;
		}
		else
		{
			if (strcmp(TestPath->GetElement(0)->GetName(), "Test") != NULL) {printf("Test Path[16] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (strcmp(TestPath->GetElement(1)->GetName(), "Keks") != NULL) {printf("Test Path[17] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
		}
		(*performedTests)++;
		delete TestPath;

		TestPath = new Path("\\");
		if (strcmp(TestPath->GetRoot(), "\\") != NULL) {printf("Test Path[18] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (TestPath->GetElementCount() != 0) {printf("Test Path[19] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		delete TestPath;

		TestPath = new Path("\\Test");
		if (strcmp(TestPath->GetRoot(), "\\") != NULL) {printf("Test Path[20] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (TestPath->GetElementCount() != 1)
		{
			printf("Test Path[21] failed!\n"); (*failedTests)++;
		}
		else
		{
			if (strcmp(TestPath->GetElement(0)->GetName(), "Test") != NULL) {printf("Test Path[22] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
		}
		(*performedTests)++;
		delete TestPath;

		TestPath = new Path("\\Test\\Keks");
		if (strcmp(TestPath->GetRoot(), "\\") != NULL) {printf("Test Path[23] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (TestPath->GetElementCount() != 2)
		{
			printf("Test Path[24] failed!\n"); (*failedTests)++;
		}
		else
		{
			if (strcmp(TestPath->GetElement(0)->GetName(), "Test") != NULL) {printf("Test Path[25] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (strcmp(TestPath->GetElement(1)->GetName(), "Keks") != NULL) {printf("Test Path[26] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
		}
		(*performedTests)++;
		delete TestPath;

		TestPath = new Path("\\\\Resource");
		if (strcmp(TestPath->GetRoot(), "\\\\Resource") != NULL) {printf("Test Path[27] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (TestPath->GetElementCount() != 0) {printf("Test Path[28] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		delete TestPath;

		TestPath = new Path("\\\\Resource\\Test");
		if (strcmp(TestPath->GetRoot(), "\\\\Resource\\") != NULL) {printf("Test Path[29] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (TestPath->GetElementCount() != 1)
		{
			printf("Test Path[30] failed!\n"); (*failedTests)++;
		}
		else
		{
			if (strcmp(TestPath->GetElement(0)->GetName(), "Test") != NULL) {printf("Test Path[31] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
		}
		(*performedTests)++;
		delete TestPath;

		TestPath = new Path("\\\\Resource\\Test\\Keks");
		if (strcmp(TestPath->GetRoot(), "\\\\Resource\\") != NULL) {printf("Test Path[32] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (TestPath->GetElementCount() != 2)
		{
			printf("Test Path[33] failed!\n"); (*failedTests)++;
		}
		else
		{
			if (strcmp(TestPath->GetElement(0)->GetName(), "Test") != NULL) {printf("Test Path[34] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (strcmp(TestPath->GetElement(1)->GetName(), "Keks") != NULL) {printf("Test Path[35] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
		}
		(*performedTests)++;
		delete TestPath;

		Path * TestPath1;
		Path * TestPath2;
		Path * RelativePath;
		char * relativePath;

		TestPath1 = new Path("foo");
		TestPath2 = new Path("bar.dat");
		RelativePath = TestPath1->CreateRelativePathTo(TestPath2);
		if (RelativePath != NULL)
			relativePath = RelativePath->ToString(false);
		else
			relativePath = NULL;
		if (relativePath == NULL || strcmp(relativePath, "..\\bar.dat") != 0)
			{printf("Test Path[36] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (relativePath != NULL) delete relativePath;
		if (RelativePath != NULL) delete RelativePath;
		delete TestPath1;
		delete TestPath2;

		TestPath1 = new Path("C:\\foo");
		TestPath2 = new Path("C:\\bar.dat");
		RelativePath = TestPath1->CreateRelativePathTo(TestPath2);
		if (RelativePath != NULL)
			relativePath = RelativePath->ToString(false);
		else
			relativePath = NULL;
		if (relativePath == NULL || strcmp(relativePath, "..\\bar.dat") != 0)
			{printf("Test Path[37] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (relativePath != NULL) delete relativePath;
		if (RelativePath != NULL) delete RelativePath;
		delete TestPath1;
		delete TestPath2;

		TestPath1 = new Path("foo");
		TestPath2 = new Path("C:\\bar.dat");
		RelativePath = TestPath1->CreateRelativePathTo(TestPath2);
		if (RelativePath != NULL)
			relativePath = RelativePath->ToString(false);
		else
			relativePath = NULL;
		if (relativePath == NULL || strcmp(relativePath, "C:\\bar.dat") != 0)
			{printf("Test Path[38] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (relativePath != NULL) delete relativePath;
		if (RelativePath != NULL) delete RelativePath;
		delete TestPath1;
		delete TestPath2;

		TestPath1 = new Path("C:\\foo");
		TestPath2 = new Path("D:\\bar.dat");
		RelativePath = TestPath1->CreateRelativePathTo(TestPath2);
		if (RelativePath != NULL)
			relativePath = RelativePath->ToString(false);
		else
			relativePath = NULL;
		if (relativePath == NULL || strcmp(relativePath, "D:\\bar.dat") != 0)
			{printf("Test Path[39] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (relativePath != NULL) delete relativePath;
		if (RelativePath != NULL) delete RelativePath;
		delete TestPath1;
		delete TestPath2;

		TestPath1 = new Path("C:\\foo");
		TestPath2 = new Path("bar.dat");
		RelativePath = TestPath1->CreateRelativePathTo(TestPath2);
		if (RelativePath != NULL)
			relativePath = RelativePath->ToString(false);
		else
			relativePath = NULL;
		if (relativePath != NULL)
			{printf("Test Path[40] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (relativePath != NULL) delete relativePath;
		if (RelativePath != NULL) delete RelativePath;
		delete TestPath1;
		delete TestPath2;

		TestPath1 = new Path("C:\\foo\\bar\\bar.dat");
		TestPath2 = new Path("C:\\foo");
		RelativePath = TestPath1->CreateRelativePathTo(TestPath2);
		if (RelativePath != NULL)
			relativePath = RelativePath->ToString(false);
		else
			relativePath = NULL;
		if (relativePath == NULL || strcmp(relativePath, "..\\..") != 0)
			{printf("Test Path[41] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (relativePath != NULL) delete relativePath;
		if (RelativePath != NULL) delete RelativePath;
		delete TestPath1;
		delete TestPath2;

		TestPath1 = new Path("C:\\foo");
		TestPath2 = new Path("C:\\foo\\bar\\bar.dat");
		RelativePath = TestPath1->CreateRelativePathTo(TestPath2);
		if (RelativePath != NULL)
			relativePath = RelativePath->ToString(false);
		else
			relativePath = NULL;
		if (relativePath == NULL || strcmp(relativePath, "bar\\bar.dat") != 0)
			{printf("Test Path[42] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (relativePath != NULL) delete relativePath;
		if (RelativePath != NULL) delete RelativePath;
		delete TestPath1;
		delete TestPath2;

	}
#endif

void Path::SetRoot(const char * root)
{
	ASSERTION_THIS;

	if (Root) {delete Root; Root = NULL;}
	if ((Root = new char[strlen(root)+1]) == NULL)
		throw EOutOfMemory::Get();

	strcpy(Root, root);
}

char * Path::ToString(bool trailingSlash)
{
	ASSERTION_THIS;

	char * path;

	int len = Elements->GetCount();
	len += strlen(Root);
	PathElement * Element = Elements->GetFirst();
	while (Element)
	{
		len += strlen(Element->GetName());
		Element = Element->GetNextElement();
	}

	if ((path = new char[len+1]) == NULL)
		throw EOutOfMemory::Get();
	strcpy(path, Root);

	Element = Elements->GetFirst();
	while (Element)
	{
		strcat(path, Element->GetName());
		if (Element != Elements->GetLast() || trailingSlash)
			strcat(path, "\\");

		Element = Element->GetNextElement();
	}

	return path;
}


// --- Class PathElement ---

PathElement::PathElement()
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	Name = NULL;
	SetName("");
}

PathElement::PathElement(const char * Name)
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	this->Name = NULL;
	SetName(Name);
}

PathElement::~PathElement()
{
	ASSERTION_THIS;

	if (Name) {delete Name; Name = NULL;}

	#ifdef _TOOLBOX_TEST
		InstanceCount--;
	#endif
}

bool PathElement::ChangeExtension(const char * newExtension)
{
	ASSERTION_THIS;

	if (Name == NULL)
		return false;

	if (newExtension[0] == '.')
		newExtension = &newExtension[1];

	char * extensionPtr;
	if ((extensionPtr = strrchr(Name, '.')) != NULL)
		extensionPtr[0] = 0;

	char * newFileName;
	if ((newFileName = new char[strlen(Name) + 1 + strlen(newExtension) + 1]) == NULL)
		throw EOutOfMemory::Get();

	sprintf(newFileName, "%s.%s", Name, newExtension);
	delete Name;
	Name = newFileName;

	return true;
}

PathElement * PathElement::Clone()
{
	ASSERTION_THIS;

	PathElement * Clone;
	if ((Clone = new PathElement(GetName())) == NULL)
		throw EOutOfMemory::Get();
	return Clone;
}

#ifdef _TOOLBOX_TEST
	int PathElement::InstanceCount = 0;
#endif

#ifdef _TOOLBOX_TEST
	void PathElement::RunTestSuite(int * performedTests, int * failedTests)
	{
	}
#endif

void PathElement::SetName(const char * name)
{
	ASSERTION_THIS;

	if (Name) {delete Name; Name = NULL;}
	if ((Name = new char[strlen(name)+1]) == NULL)
		throw EOutOfMemory::Get();
	strcpy(Name, name);

	while (Name[0] == '\\')
		memmove(Name, &Name[1], strlen(&Name[1]) + 1);
	while (Name[0] && Name[strlen(Name)-1] == '\\')
		Name[strlen(Name)-1] = 0;
}
