/*
 * This file is part of the ToolBox.
 * Copyright Thomas Jacob.
 *
 * READ README.TXT BEFORE USE!!
 */


// Main header file
#include <ToolBox\Headers.h>


// --- Class Edge ---

Edge::Edge(class Graph * Graph, Vertex * FromVertex, Vertex * ToVertex)
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	Data = NULL;
	this->FromVertex = FromVertex;
	this->Graph = Graph;
	this->ToVertex = ToVertex;
}

Edge::~Edge()
{
	ASSERTION_THIS;

	for (int i=0; i<Graph->GetVertices()->GetCount(); i++)
		Graph->GetVertices()->Get(i)->GetEdges()->Unlink(this);

	Graph->GetEdges()->Unlink(this);

	#ifdef _TOOLBOX_TEST
		InstanceCount--;
	#endif
}

#ifdef _TOOLBOX_TEST
	int Edge::InstanceCount = 0;
#endif

#ifdef _TOOLBOX_TEST
	void Edge::RunTestSuite(int * performedTests, int * failedTests)
	{
	}
#endif


// --- Class EdgeArray ---

EdgeArray::EdgeArray()
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif
}

EdgeArray::~EdgeArray()
{
	ASSERTION_THIS;

	#ifdef _TOOLBOX_TEST
		InstanceCount--;
	#endif
}

#ifdef _TOOLBOX_TEST
	int EdgeArray::InstanceCount = 0;
#endif

#ifdef _TOOLBOX_TEST
	void EdgeArray::RunTestSuite(int * performedTests, int * failedTests)
	{
	}
#endif


// --- Class Graph ---

Graph::Graph()
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	if ((Edges = new EdgeArray) == NULL)
		throw EOutOfMemory::Get();

	if ((Vertices = new VertexArray) == NULL)
	{
		delete Edges; Edges = NULL;
		throw EOutOfMemory::Get();
	}
}

Graph::~Graph()
{
	ASSERTION_THIS;

	if (Edges) {delete Edges; Edges = NULL;}
	if (Vertices) {delete Vertices; Vertices = NULL;}

	#ifdef _TOOLBOX_TEST
		InstanceCount--;
	#endif
}

Vertex * Graph::AddVertex()
{
	ASSERTION_THIS;

	Vertex * NewVertex;
	if ((NewVertex = new Vertex(this)) == NULL)
		throw EOutOfMemory::Get();
	Vertices->Append(NewVertex);
	return NewVertex;
}

#ifdef _TOOLBOX_TEST
	int Graph::InstanceCount = 0;
#endif

#ifdef _TOOLBOX_TEST
	void Graph::RunTestSuite(int * performedTests, int * failedTests)
	{
	}
#endif


// --- Class Vertex ---

Vertex::Vertex(class Graph * Graph)
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	Data = NULL;
	if ((Edges = new EdgeArray) == NULL)
		throw EOutOfMemory::Get();
	this->Graph = Graph;
}

Vertex::~Vertex()
{
	ASSERTION_THIS;

	for (int i=0; i<Graph->GetEdges()->GetCount(); i++)
		if (Graph->GetEdges()->Get(i)->GetToVertex() == this)
			Graph->GetEdges()->Delete(i);

	Graph->GetVertices()->Unlink(this);

	if (Edges) {delete Edges; Edges = NULL;}

	#ifdef _TOOLBOX_TEST
		InstanceCount--;
	#endif
}

Edge * Vertex::AddEdge(Vertex * ToVertex)
{
	ASSERTION_THIS;

	Edge * NewEdge;
	if ((NewEdge = new Edge(Graph, this, ToVertex)) == NULL)
		throw EOutOfMemory::Get();
	Edges->Append(NewEdge);
	Graph->GetEdges()->Append(NewEdge);
	return NewEdge;
}

bool Vertex::DeleteEdge(Vertex * ToVertex)
{
	ASSERTION_THIS;

	for (int i=0; i<Edges->GetCount(); i++)
		if (Edges->Get(i)->GetToVertex() == ToVertex)
		{
			Edges->Delete(i);
			return true;
		}

	return false;
}

#ifdef _TOOLBOX_TEST
	int Vertex::InstanceCount = 0;
#endif

#ifdef _TOOLBOX_TEST
	void Vertex::RunTestSuite(int * performedTests, int * failedTests)
	{
	}
#endif


// --- Class VertexArray ---

VertexArray::VertexArray()
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif
}

VertexArray::~VertexArray()
{
}

#ifdef _TOOLBOX_TEST
	int VertexArray::InstanceCount = 0;
#endif

#ifdef _TOOLBOX_TEST
	void VertexArray::RunTestSuite(int * performedTests, int * failedTests)
	{
	}
#endif
