/*
 * This file is part of the ToolBox.
 * Copyright Thomas Jacob.
 *
 * READ README.TXT BEFORE USE!!
 */


#ifndef __TOOLBOX_LINKEDLIST_INL
#define __TOOLBOX_LINKEDLIST_INL


// --- Class ContainingList ---

template <class ITEMTYPE> ContainingList<ITEMTYPE>::ContainingList()
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	First = NULL;
	Last  = NULL;
	Count = 0;
}

template <class ITEMTYPE> ContainingList<ITEMTYPE>::~ContainingList()
{
	ASSERTION_THIS;

	DeleteAll();

	#ifdef _TOOLBOX_TEST
		InstanceCount--;
	#endif
}

template <class ITEMTYPE> void ContainingList<ITEMTYPE>::Append(ITEMTYPE * Item)
{
	ASSERTION_THIS;

	Item->PrevItem = Last;

	if (Last)
		Last->NextItem = Item;
	else
		First = Item;
	Last = Item;

	Count++;
}

template <class ITEMTYPE> int ContainingList<ITEMTYPE>::Compare(const ITEMTYPE * Item1, const ITEMTYPE * Item2) const
{
	return Item1==Item2 ? 0 : (Item1>Item2 ? 1 : -1);
}

template <class ITEMTYPE> bool ContainingList<ITEMTYPE>::Delete(long nr)
{
	ASSERTION_THIS;

	ITEMTYPE * DeleteItem;

	if ((DeleteItem = ContainingList<ITEMTYPE>::Unlink(nr)) != NULL)
	{
		delete (ITEMTYPE *) DeleteItem;
		return true;
	}
	else
		return false;
}

template <class ITEMTYPE> bool ContainingList<ITEMTYPE>::Delete(ITEMTYPE * Item)
{
	ASSERTION_THIS;

	ITEMTYPE * DeleteItem;

	if ((DeleteItem = ContainingList<ITEMTYPE>::Unlink(Item)) != NULL)
	{
		delete (ITEMTYPE *) DeleteItem;
		return true;
	}
	else
		return false;
}

template <class ITEMTYPE> bool ContainingList<ITEMTYPE>::Delete(ITEMTYPE * Item,
	int (* compare)(const ITEMTYPE * Item1, const ITEMTYPE * Item2))
{
	ASSERTION_THIS;

	ITEMTYPE * DeleteItem;

	if ((DeleteItem = ContainingList<ITEMTYPE>::Unlink(compare, Item)) != NULL)
	{
		delete (ITEMTYPE *) DeleteItem;
		return true;
	}
	else
		return false;
}

template <class ITEMTYPE> void ContainingList<ITEMTYPE>::DeleteAll()
{
	ASSERTION_THIS;

	while (Last)
		ContainingList<ITEMTYPE>::DeleteLast();
}

template <class ITEMTYPE> inline void ContainingList<ITEMTYPE>::DeleteDirectly(ITEMTYPE * Item)
{
	ASSERTION_THIS;
	delete ContainingList<ITEMTYPE>::UnlinkDirectly(Item);
}

template <class ITEMTYPE> inline bool ContainingList<ITEMTYPE>::DeleteFirst()
{
	ASSERTION_THIS;

	ITEMTYPE * DeleteItem;

	if ((DeleteItem = ContainingList<ITEMTYPE>::UnlinkFirst()) != NULL)
	{
		delete (ITEMTYPE *) DeleteItem;
		return true;
	}
	else
		return false;
}

template <class ITEMTYPE> bool ContainingList<ITEMTYPE>::DeleteLast()
{
	ASSERTION_THIS;

	ITEMTYPE * DeleteItem;

	if ((DeleteItem = ContainingList<ITEMTYPE>::UnlinkLast()) != NULL)
	{
		delete (ITEMTYPE *) DeleteItem;
		return true;
	}
	else
		return false;
}

template <class ITEMTYPE> ITEMTYPE * ContainingList<ITEMTYPE>::Get(long nr) const
{
	ASSERTION_THIS;

	ITEMTYPE * CurrentItem;

	for (CurrentItem = First; CurrentItem, nr>0; CurrentItem = CurrentItem->NextItem, nr--);
	return CurrentItem;
}

template <class ITEMTYPE> inline ITEMTYPE * ContainingList<ITEMTYPE>::Get(const ITEMTYPE * SearchItem) const
{
	ASSERTION_THIS;
	return Get(NULL, SearchItem);
}

template <class ITEMTYPE> ITEMTYPE * ContainingList<ITEMTYPE>::Get(const ITEMTYPE * SearchItem,
	int (* compare)(const ITEMTYPE * Item1, const ITEMTYPE * Item2)) const
{
	ASSERTION_THIS;

	ITEMTYPE * CurrentItem;

	for (CurrentItem = First; CurrentItem; CurrentItem = CurrentItem->NextItem)
		if (compare
			? compare(CurrentItem, SearchItem)==0
			: Compare(CurrentItem, SearchItem)==0)
			break;

	return CurrentItem;
}

template <class ITEMTYPE> inline long ContainingList<ITEMTYPE>::GetCount() const
{
	ASSERTION_THIS;
	return Count;
}

template <class ITEMTYPE> inline ITEMTYPE * ContainingList<ITEMTYPE>::GetFirst() const
{
	ASSERTION_THIS;
	return First;
}

template <class ITEMTYPE> inline ITEMTYPE * ContainingList<ITEMTYPE>::GetLast() const
{
	ASSERTION_THIS;
	return Last;
}

template <class ITEMTYPE> void ContainingList<ITEMTYPE>::Insert(ITEMTYPE * Item, long nr)
{
	ASSERTION_THIS;

	ITEMTYPE * InsertBeforeItem;

	for (InsertBeforeItem = First; InsertBeforeItem && nr>0;
		InsertBeforeItem = InsertBeforeItem->NextItem, nr--);
	if (nr > 0)
		return;

	if (InsertBeforeItem == NULL)
	{
		Item->PrevItem = Last;

		if (Last)
			Last->NextItem = Item;
		else
			First = Item;
		Last = Item;
	}
	else
	{
		Item->PrevItem = InsertBeforeItem->PrevItem;
		Item->NextItem = InsertBeforeItem;

		if (InsertBeforeItem->PrevItem)
			InsertBeforeItem->PrevItem->NextItem = Item;
		else
			First = Item;
		InsertBeforeItem->PrevItem = Item;
	}

	Count++;
}

#ifdef _TOOLBOX_TEST
	template <class ITEMTYPE> int ContainingList<ITEMTYPE>::InstanceCount = 0;
#endif

template <class ITEMTYPE> inline bool ContainingList<ITEMTYPE>::IsEmpty() const
{
	ASSERTION_THIS;
	return Count<=0;
}

template <class ITEMTYPE> inline void ContainingList<ITEMTYPE>::Prepend(ITEMTYPE * Item)
{
	ASSERTION_THIS;

	Item->NextItem = First;

	if (First)
		First->PrevItem = Item;
	else
		Last = Item;
	First = Item;

	Count++;
}

template <class ITEMTYPE> void ContainingList<ITEMTYPE>::Reverse()
{
	ASSERTION_THIS;

	ITEMTYPE * CurrentItem, *Merk;

	for (CurrentItem = First; CurrentItem; CurrentItem = CurrentItem->NextItem)
	{
		Merk = CurrentItem->NextItem;
		CurrentItem->NextItem = CurrentItem->PrevItem;
		CurrentItem->PrevItem = Merk;
	}

	Merk = First;
	First = Last;
	Last = Merk;
}

#ifdef _TOOLBOX_TEST
template <class ITEMTYPE> void ContainingList<ITEMTYPE>::RunTestSuite(int * performedTests, int * failedTests)
{
}
#endif

template <class ITEMTYPE> inline void ContainingList<ITEMTYPE>::Sort()
{
	ASSERTION_THIS;
	Sort(NULL);
}

template <class ITEMTYPE> void ContainingList<ITEMTYPE>::Sort(int (* compare)(const ITEMTYPE * Item1, const ITEMTYPE * Item2))
{
	ASSERTION_THIS;

	ITEMTYPE * OldHead = First, *SearchItem, *MinItem;

	First = NULL; Last = NULL; Count=0;
	while (OldHead)
	{
		MinItem = OldHead;
		for (SearchItem = OldHead->NextItem; SearchItem; SearchItem = SearchItem->NextItem)
			if ((compare ? compare(SearchItem, MinItem) : Compare(SearchItem, MinItem)) < 0)
				MinItem = SearchItem;

		if (MinItem->PrevItem)
			MinItem->PrevItem->NextItem = MinItem->NextItem;
		else
			OldHead = MinItem->NextItem;

		if (MinItem->NextItem)
			MinItem->NextItem->PrevItem = MinItem->PrevItem;

		MinItem->NextItem = NULL;
		MinItem->PrevItem = NULL;
		ContainingList<ITEMTYPE>::Append(MinItem);
	}
}

template <class ITEMTYPE> ITEMTYPE * ContainingList<ITEMTYPE>::Unlink(long nr)
{
	ASSERTION_THIS;

	ITEMTYPE * CurrentItem;

	for (CurrentItem = First; nr>0 && CurrentItem; CurrentItem = CurrentItem->NextItem, nr--);
	if (!CurrentItem)
		return NULL;

	if (CurrentItem->PrevItem)
		CurrentItem->PrevItem->NextItem = CurrentItem->NextItem;
	else
		First = CurrentItem->NextItem;

	if (CurrentItem->NextItem)
		CurrentItem->NextItem->PrevItem = CurrentItem->PrevItem;
	else
		Last = CurrentItem->PrevItem;

	Count--;
	CurrentItem->PrevItem = NULL;
	CurrentItem->NextItem = NULL;

	return CurrentItem;
}

template <class ITEMTYPE> ITEMTYPE * ContainingList<ITEMTYPE>::Unlink(ITEMTYPE * Item)
{
	ASSERTION_THIS;

	ITEMTYPE * CurrentItem;

	for (CurrentItem = First; CurrentItem; CurrentItem = CurrentItem->NextItem)
		if (Compare(CurrentItem, Item)==0)
		{
			if (CurrentItem->PrevItem)
				CurrentItem->PrevItem->NextItem = CurrentItem->NextItem;
			else
				First = CurrentItem->NextItem;

			if (CurrentItem->NextItem)
				CurrentItem->NextItem->PrevItem = CurrentItem->PrevItem;
			else
				Last = CurrentItem->PrevItem;

			Count--;
			CurrentItem->PrevItem = NULL;
			CurrentItem->NextItem = NULL;

			return Item;
		}

	return NULL;
}

template <class ITEMTYPE> ITEMTYPE * ContainingList<ITEMTYPE>::Unlink(ITEMTYPE * Item,
	int (* compare)(const ITEMTYPE * Item1, const ITEMTYPE * Item2))
{
	ASSERTION_THIS;

	ITEMTYPE * CurrentItem;

	for (CurrentItem = First; CurrentItem; CurrentItem = CurrentItem->NextItem)
		if (compare(CurrentItem, Item)==0)
		{
			if (CurrentItem->PrevItem)
				CurrentItem->PrevItem->NextItem = CurrentItem->NextItem;
			else
				First = CurrentItem->NextItem;

			if (CurrentItem->NextItem)
				CurrentItem->NextItem->PrevItem = CurrentItem->PrevItem;
			else
				Last = CurrentItem->PrevItem;

			Count--;
			CurrentItem->PrevItem = NULL;
			CurrentItem->NextItem = NULL;

			return Item;
		}

	return NULL;
}

template <class ITEMTYPE> void ContainingList<ITEMTYPE>::UnlinkAll()
{
	ASSERTION_THIS;

	while (Last)
		ContainingList<ITEMTYPE>::UnlinkLast();
}

template <class ITEMTYPE> ITEMTYPE * ContainingList<ITEMTYPE>::UnlinkDirectly(ITEMTYPE * Item)
{
	ASSERTION_THIS;

	if (Item->PrevItem)
		Item->PrevItem->NextItem = Item->NextItem;
	else
		First = Item->NextItem;

	if (Item->NextItem)
		Item->NextItem->PrevItem = Item->PrevItem;
	else
		Last = Item->PrevItem;

	Count--;
	Item->PrevItem = NULL;
	Item->NextItem = NULL;

	return Item;
}

template <class ITEMTYPE> inline ITEMTYPE * ContainingList<ITEMTYPE>::UnlinkFirst()
{
	ASSERTION_THIS;
	return ContainingList<ITEMTYPE>::Unlink((long) 0);
}

template <class ITEMTYPE> ITEMTYPE * ContainingList<ITEMTYPE>::UnlinkLast()
{
	ASSERTION_THIS;

	ITEMTYPE * Item;

	if (!Last)
		return NULL;

	if (Last->PrevItem)
		Last->PrevItem->NextItem = NULL;
	else
		First = NULL;

	Item = Last;
	Last = Last->PrevItem;
	Count--;
	Item->PrevItem = NULL;
	Item->NextItem = NULL;

	return Item;
}


// --- Class ContainingListItem ---

template <class ITEMTYPE> ContainingListItem<ITEMTYPE>::ContainingListItem()
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	NextItem = NULL;
	PrevItem = NULL;
}

template <class ITEMTYPE> ContainingListItem<ITEMTYPE>::~ContainingListItem()
{
	ASSERTION_THIS;

	#ifdef _TOOLBOX_TEST
		InstanceCount--;
	#endif
}

template <class ITEMTYPE> inline ITEMTYPE * ContainingListItem<ITEMTYPE>::GetNext() const
{
	ASSERTION_THIS;
	return NextItem;
}

template <class ITEMTYPE> inline ITEMTYPE * ContainingListItem<ITEMTYPE>::GetPrev() const
{
	ASSERTION_THIS;
	return PrevItem;
}

#ifdef _TOOLBOX_TEST
	template <class ITEMTYPE> int ContainingListItem<ITEMTYPE>::InstanceCount = 0;
#endif

#ifdef _TOOLBOX_TEST
template <class ITEMTYPE> void ContainingListItem<ITEMTYPE>::RunTestSuite(int * performedTests, int * failedTests)
{
}
#endif


// --- Class PointeredListItem ---

inline void * PointeredListItem::GetData() const
{
	ASSERTION_THIS;
	return Data;
}

inline void PointeredListItem::SetData(void * Data)
{
	ASSERTION_THIS;
	this->Data = Data;
}


#endif
