/*
 * This file is part of the ToolBox.
 * Copyright Thomas Jacob.
 *
 * READ README.TXT BEFORE USE!!
 */


#ifndef __TOOLBOX_LOGGER_H
#define __TOOLBOX_LOGGER_H


/**
 * Log level: Show no messages (logger is off).
 */
#define LOG_NEVER 127

/**
 * The logging session separator.
 */
#define LOG_SESSIONSEPARATOR "\n-------------------------------------------------------------------------------\n"

/**
 * As a log type: Message for debugging only
 * (it is in no case a problem, and is only
 * interesting for the owning developer).<br>
 * As a log level: Show all messages of debug type or higher.
 */
#define LOG_DEBUG 1

/**
 * As a log type: Message is a note
 * (it is in no case a problem, but may
 * be interesting for all developers).<br>
 * As a log level: Show all messages of note type or higher.
 */
#define LOG_INFO 2

/**
 * As a log type: Message is a warning
 * (situation may become a problem).<br>
 * As a log level: Show all messages of warning type or higher.
 */
#define LOG_WARN 3

/**
 * As a log type: Message is an error
 * (it is in every case a problem).<br>
 * As a log level: Show all messages of error type or higher.
 */
#define LOG_ERROR 4

/**
 * As a log type: Message is an exception
 * (it is in every case a problem).<br>
 * As a log level: Show all messages of exception type or higher.
 */
#define LOG_EXCEPTION 5

/**
 * As a log type: Message is fatal
 * (it is a problem that causes program stop).<br>
 * As a log level: Show all messages of fatal type or higher.
 */
#define LOG_FATAL 6


// Macro functions

/**
 * Macro for the debug logging with 0 parameters.
 */
#define LOGPRINTF0(type, format) \
	(Logger::Get()!=NULL ? Logger::Get()->Printf(__FILE__, __LINE__, (type), (format)) \
		: 0)
/**
 * Macro for the debug logging with 1 parameter.
 */
#define LOGPRINTF1(type, format, p1) \
	(Logger::Get()!=NULL ? Logger::Get()->Printf(__FILE__, __LINE__, (type), (format), \
		(p1)) : 0)
/**
 * Macro for the debug logging with 2 parameters.
 */
#define LOGPRINTF2(type, format, p1, p2) \
	(Logger::Get()!=NULL ? Logger::Get()->Printf(__FILE__, __LINE__, (type), (format), \
		(p1), (p2)) : 0)
/**
 * Macro for the debug logging with 3 parameters.
 */
#define LOGPRINTF3(type, format, p1, p2, p3) \
	(Logger::Get()!=NULL ? Logger::Get()->Printf(__FILE__, __LINE__, (type), (format), \
		(p1), (p2), (p3)) : 0)
/**
 * Macro for the debug logging with 4 parameters.
 */
#define LOGPRINTF4(type, format, p1, p2, p3, p4) \
	(Logger::Get()!=NULL ? Logger::Get()->Printf(__FILE__, __LINE__, (type), (format), \
		(p1), (p2), (p3), (p4)) : 0)
/**
 * Macro for the debug logging with 5 parameters.
 */
#define LOGPRINTF5(type, format, p1, p2, p3, p4, p5) \
	(Logger::Get()!=NULL ? Logger::Get()->Printf(__FILE__, __LINE__, (type), (format), \
		(p1), (p2), (p3), (p4), (p5)) : 0)
/**
 * Macro for the debug logging with 6 parameters.
 */
#define LOGPRINTF6(type, format, p1, p2, p3, p4, p5, p6) \
	(Logger::Get()!=NULL ? Logger::Get()->Printf(__FILE__, __LINE__, (type), (format), \
		(p1), (p2), (p3), (p4), (p5), (p6)) : 0)
/**
 * Macro for the debug logging with 7 parameters.
 */
#define LOGPRINTF7(type, format, p1, p2, p3, p4, p5, p6, p7) \
	(Logger::Get()!=NULL ? Logger::Get()->Printf(__FILE__, __LINE__, (type), (format), \
		(p1), (p2), (p3), (p4), (p5), (p6), (p7)) : 0)
/**
 * Macro for the debug logging with 8 parameters.
 */
#define LOGPRINTF8(type, format, p1, p2, p3, p4, p5, p6, p7, p8) \
	(Logger::Get()!=NULL ? Logger::Get()->Printf(__FILE__, __LINE__, (type), (format), \
		(p1), (p2), (p3), (p4), (p5), (p6), (p7), (p8)) : 0)
/**
 * Macro for the debug logging with 9 parameters.
 */
#define LOGPRINTF9(type, format, p1, p2, p3, p4, p5, p6, p7, p8, p9) \
	(Logger::Get()!=NULL ? Logger::Get()->Printf(__FILE__, __LINE__, (type), (format), \
		(p1), (p2), (p3), (p4), (p5), (p6), (p7), (p8), (p9)) : 0)
/**
 * Macro for the debug logging with 10 parameters.
 */
#define LOGPRINTF10(type, format, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10) \
	(Logger::Get()!=NULL ? Logger::Get()->Printf(__FILE__, __LINE__, (type), (format), \
		(p1), (p2), (p3), (p4), (p5), (p6), (p7), (p8), (p9), (p10)) : 0)


namespace toolbox
{
	/**
	 * The Logger class provides debugging functionality
	 * for output messages into a separate file, the log file.
	 * @author Thomas Jacob
	 */
	class Logger
	{
		private:

			/**
			 * The log file.
			 */
			FILE * File;

			/**
			 * The log file name.
			 */
			char * FileName;

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

			/**
			 * The current log level (logging threshold).
			 */
			int LogLevel;

			/**
			 * Whether a command line option for the log level is set.
			 */
			bool OptionSet;

			/**
			 * The logger singleton.
			 */
			static Logger Singleton;

			/**
			 * Creates a new logger.
			 */
			Logger();

			/**
			 * Closes the log file, if it is open.
			 */
			void Close();

			/**
			 * Opens the log file, if it is not already open.
			 */
			bool Open();

			/**
			 * The tick count on logger startup.
			 */
			unsigned long StartTickCount;

			/**
			 * Writes the date/time and type prefix.
			 * @param type The log type.
			 */
			void WriteTimeStampPrefix(int type);

			/**
			 * Writes the date/time prefix.
			 */
			void WriteTimeStampPrefix();

		public:

			/**
			 * Destroys the logger.
			 */
			~Logger();

			/**
			 * Returns the logger singleton.
			 */
			static inline Logger * Get();

			/**
			 * Writes a log message into the log file.
			 * Source name and line number are also added automatically.
			 * Another feature of this method is the ellipse to provide
			 * flexibility like the printf functions of C.
			 * Do not call this method directly, but use the LOGPRINTFx
			 * macros, that automatically add source name and line number.
			 * @param source The source file name of the caller.
			 * @param lineNumber The source line number of the caller.
			 * @param type The log message type.
			 * @param format The output format according to the printf function.
			 * @param ... The ellipse according to the printf function.
			 * @return Always zero. It is only required by the macros.
			 */
			short Printf(const char * source, long lineNumber, int type,
				const char * format, ...);

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};
}


#endif
