/*
 * This file is part of the ToolBox.
 * Copyright Thomas Jacob.
 *
 * READ README.TXT BEFORE USE!!
 */


// Main header file
#include <ToolBox\Headers.h>


// --- Class Simulation ---

Simulation::Simulation()
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	Clock = 0;
	if ((Events = new ArrayList<SimulationEvent>) == NULL)
		throw EOutOfMemory::Get();
	HasEndedFlag = true;
	if ((Objects = new ArrayList<SimulationObject>) == NULL)
	{
		delete Events; Events = NULL;
		throw EOutOfMemory::Get();
	}

	Reset();
}

Simulation::~Simulation()
{
	ASSERTION_THIS;

	delete Events;
	delete Objects;

	#ifdef _TOOLBOX_TEST
		InstanceCount--;
	#endif
}

void Simulation::AddEvent(SimulationEvent * Event, long time)
{
	ASSERTION_THIS;

	Event->Simulation = this;
	Event->Time = time;

	for (int eventInsertPosition = 0; eventInsertPosition < Events->GetCount(); eventInsertPosition++)
		if (Events->Get(eventInsertPosition)->GetTime() > time)
			break;

	if (eventInsertPosition >= Events->GetCount())
		Events->Append(Event);
	else
		Events->Insert(Event, eventInsertPosition);
}

void Simulation::AddObject(SimulationObject * Object)
{
	ASSERTION_THIS;

	Object->Simulation = this;
	Objects->Append(Object);
}

void Simulation::DeleteEvent(int eventNr)
{
	ASSERTION_THIS;

	if (eventNr >= 0 && eventNr < Events->GetCount())
		Events->Delete(eventNr);
}

void Simulation::DeleteEvent(SimulationEvent * Event)
{
	ASSERTION_THIS;

	for (int eventNr = 0; eventNr < Events->GetCount(); eventNr++)
		if (Events->Get(eventNr) == Event)
			break;

	if (eventNr < Events->GetCount())
		Events->Delete(eventNr);
}

void Simulation::DeleteObject(int objectNr)
{
	ASSERTION_THIS;

	if (objectNr >= 0 && objectNr < Objects->GetCount())
		Objects->Delete(objectNr);
}

void Simulation::DeleteObject(SimulationObject * Object)
{
	ASSERTION_THIS;

	for (int objectNr = 0; objectNr < Objects->GetCount(); objectNr++)
		if (Objects->Get(objectNr) == Object)
			break;

	if (objectNr < Objects->GetCount())
		Objects->Delete(objectNr);
}

SimulationEvent * Simulation::GetEvent(int eventNr)
{
	ASSERTION_THIS;

	if (eventNr >= 0 && eventNr < Events->GetCount())
		return Events->Get(eventNr);
	else
		return NULL;
}

SimulationObject * Simulation::GetObject(int objectNr)
{
	ASSERTION_THIS;

	if (objectNr >= 0 && objectNr < Objects->GetCount())
		return Objects->Get(objectNr);
	else
		return NULL;
}

#ifdef _TOOLBOX_TEST
	int Simulation::InstanceCount = 0;
#endif

void Simulation::Reset()
{
	ASSERTION_THIS;

	Events->DeleteAll();
	Objects->DeleteAll();
}

void Simulation::Run()
{
	ASSERTION_THIS;

	HasEndedFlag = false;
	while (!Events->IsEmpty() && !HasEndedFlag)
	{
		SimulationEvent * CurrentEvent = Events->UnlinkFirst();
		Clock = CurrentEvent->GetTime();
		CurrentEvent->Handle();
		delete CurrentEvent;
	}
	HasEndedFlag = true;
}

void Simulation::Run(long endTime)
{
	ASSERTION_THIS;

	SimulationEndEvent * EndEvent;
	if ((EndEvent = new SimulationEndEvent) == NULL)
		throw EOutOfMemory::Get();

	AddEvent(EndEvent, endTime);
	Run();
}

#ifdef _TOOLBOX_TEST
	void Simulation::RunTestSuite(int * performedTests, int * failedTests)
	{
	}
#endif


// --- Class SimulationEndEvent ---

SimulationEndEvent::SimulationEndEvent()
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif
}

SimulationEndEvent::~SimulationEndEvent()
{
	ASSERTION_THIS;

	#ifdef _TOOLBOX_TEST
		InstanceCount--;
	#endif
}

#ifdef _TOOLBOX_TEST
	int SimulationEndEvent::InstanceCount = 0;
#endif

void SimulationEndEvent::Handle()
{
	ASSERTION_THIS;
	GetSimulation()->End();
}

#ifdef _TOOLBOX_TEST
	void SimulationEndEvent::RunTestSuite(int * performedTests, int * failedTests)
	{
	}
#endif


// --- Class SimulationEvent ---

SimulationEvent::SimulationEvent()
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	Simulation = NULL;
	Time = 0;
}

SimulationEvent::~SimulationEvent()
{
	ASSERTION_THIS;

	#ifdef _TOOLBOX_TEST
		InstanceCount--;
	#endif
}

#ifdef _TOOLBOX_TEST
	int SimulationEvent::InstanceCount = 0;
#endif

void SimulationEvent::Handle()
{
	ASSERTION_THIS;
}

#ifdef _TOOLBOX_TEST
	void SimulationEvent::RunTestSuite(int * performedTests, int * failedTests)
	{
	}
#endif


// --- Class SimulationObject ---

SimulationObject::SimulationObject()
{
	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	Simulation = NULL;
}

SimulationObject::~SimulationObject()
{
	ASSERTION_THIS;

	#ifdef _TOOLBOX_TEST
		InstanceCount--;
	#endif
}

#ifdef _TOOLBOX_TEST
	int SimulationObject::InstanceCount = 0;
#endif

#ifdef _TOOLBOX_TEST
	void SimulationObject::RunTestSuite(int * performedTests, int * failedTests)
	{
	}
#endif
